// GUIBASE.H : a base class for all graphical user interfaces.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef GUIBASE_H
#define GUIBASE_H

#ifdef ENABLE_GUI

class bs_gui;
class color_mode;

////////////////////////////////////////////////////////////////////////////////////////////////////

class view_window;		// window.h
class window_factory;		// window.h

class dummy_object;		// objects.h
class smart_object;		// objects.h

class camera;			// viewing.h
class light;			// viewing.h

class transformer;		// viewing.h

#include "mdlbase.h"

#include <vector>

#ifdef TARGET3
#include <gtk/gtk.h>
#endif	// TARGET3

#define SB_SIZE 512		// the OGL selection buffer size.

////////////////////////////////////////////////////////////////////////////////////////////////////

// should the "gui"-classes be split into separate classes for graphics and event handling???
// graphics handler -> "grh" and event hander -> "evh" ???????????????

// why??? with this design it would be possible to present different models (for example, a MM
// model and a QM model) in a single window!!!!?!?! but would it be useful????????

// would that be enough if we just allow the Render() -function be separate from this class???

class bs_gui : virtual public bs_mdl_simple
{
	protected:
	
	vector<camera *> camera_vector;
	vector<view_window *> view_window_vector;
	
	// in TARGET3 there can be several GUI's running simultaneously,
	// so we must make sure that the display list ID's are unique...
	
	static iGLu list_counter;
	
	public:
	
	enum rmode { Normal = 0, Transform1 = 1, Transform2 = 2 };
	
	window_factory * factory;
	
	vector<light *> light_vector;
	vector<smart_object *> object_vector;
	dummy_object * selected_object;
	
	iGLu * select_buffer;
	
#ifdef TARGET3

	GtkWidget * popupmenu;
	
#endif	// TARGET3

	public:
	
	bs_gui(window_factory &, ostream *);
	virtual ~bs_gui(void);
	
	// some utility functions.
	
	virtual const char * GetType(void) = 0;
	virtual color_mode * GetDefaultColorMode(void) = 0;
	
	virtual void SelectAll(void) = 0;
	virtual void InvertSelection(void) = 0;
		
	iGLu GetDisplayListIDs(iGLu);
	void DeleteDisplayLists(iGLu, iGLu);
	
	// some light-related functions:
	
	i32s IsLight(const dummy_object *);
	bool SelectLight(const dummy_object *);
	
	bool AddGlobalLight(light *);
	bool AddLocalLight(light *, camera *);
	bool RemoveLight(dummy_object *);
	
	iGLs CountGlobalLights(void);
	iGLs CountLocalLights(camera *);
	
	bool SetupLights(camera *);
	
	void SetGlobalLightNumbers(void);
	void SetLocalLightNumbers(camera *);
	
	// some object-related functions:
	
	i32s IsObject(const dummy_object *);
	bool SelectObject(const dummy_object *);
	
	void AddObject(smart_object *);
	bool RemoveObject(dummy_object *);
	
	// some camera/window functions:
	
	virtual view_window * AddCamera(fGL);
	virtual view_window * AddViewWindow(camera *);
	virtual bool RemoveViewWindow(view_window *);
	
	void UpdateAllWindowTitles(void);
	
	void UpdateAllViewWindows(bool = false);
	void UpdateViewWindows(camera *, bool = false);
	void UpdateViewWindow(view_window *, bool = false);
	
	// remember to assign the selection buffer for all models in model::InitGL() !!!
	
	virtual void InitGL(void) = 0;
	virtual void Render(view_window *, rmode) = 0;
	
	// model::Center() should center the transformer object to the center of all
	// selected atoms, and shift the center of all selected atoms to the origo.
	
	// model::Transform() should get the transformation matrix from the transformer
	// object and use it to transform all selected atoms/bonds/whatever.
	
	virtual void Center(transformer *) = 0;
	virtual void Transform(transformer *) = 0;
	
	virtual void DrawEvent(view_window *, vector<iGLu> &) = 0;
	virtual void EraseEvent(view_window *, vector<iGLu> &) = 0;
	virtual void SelectEvent(view_window *, vector<iGLu> &) = 0;
	
	public:
	
#ifdef TARGET2

	virtual void KeyEvent(view_window *, char, int, int) = 0;
	
#endif	// TARGET2

#ifdef TARGET3

	virtual void PopupEvent(view_window *, int) = 0;
	
#endif	// TARGET3

};

////////////////////////////////////////////////////////////////////////////////////////////////////

class color_mode
{
	protected:
	
// this is just an interface definition -> there is no relevant common data?!?!?
// this is just an interface definition -> there is no relevant common data?!?!?
// this is just an interface definition -> there is no relevant common data?!?!?

	public:
	
	color_mode(void) { }
	virtual ~color_mode(void) { }
	
// is this (void *)-method really the only possibility to give a virtual function different
// parameters without the "hiding problem"???????????????????

// should be [void *, i32s CRD_SET, fGL_a3] -> crd-set can be included in coloring???
// should be [void *, i32s CRD_SET, fGL_a3] -> crd-set can be included in coloring???
// should be [void *, i32s CRD_SET, fGL_a3] -> crd-set can be included in coloring???

	virtual void GetColor(const void *, fGL_a3 &) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI
#endif	// GUIBASE_H

// eof
